@@ -85,11 +85,12 @@ import gnu.classpath.tools.MalformedInputEvent;
 	 }
       }
 
-      int process(Parser parser, char[] source, int startIndex, int endIndex) throws ParseException, IOException {
-
-	 parser.classClosed();
-	 return endIndex;
-      }
+     int process(Parser parser, char[] source, int startIndex, int endIndex) 
+       throws ParseException, IOException 
+     {
+       parser.classClosed();
+       return endIndex;
+     }
    }
 
    class CommentComponent extends SourceComponent {
@@ -541,6 +542,25 @@ import gnu.classpath.tools.MalformedInputEvent;
                   }
                }
                break;
+            case '@':  // annotation
+               index += 1;
+               while(index<source.length &&  Character.isJavaIdentifierPart(source[index])) {
+                   ++ index;
+               }
+               if (index<source.length && source[index]=='(') {
+		   int parLevel = 1;
+                   index += 1;
+                   while (index<source.length && parLevel>0) { 
+		       if (source[index] == '(')
+                          ++ parLevel;
+		       if (source[index] == ')')
+			  -- parLevel;
+                       ++ index;
+		       if (parLevel==0)
+		           break;
+                   } 
+               }
+               break;
 	    case '=':  // field
 	    case ';':  // field
 	    case '(':  // function
@@ -563,7 +583,25 @@ import gnu.classpath.tools.MalformedInputEvent;
 	 if (parser.getAddComments())
 	    parser.ctx.classDoc.setRawCommentText(parser.getLastComment());
 	 parser.setLastComment(null);
-
+	 if (parser.ctx.classDoc.isEnum())
+	   {
+	     for (int a = endIndex; a < source.length; ++a)
+	     {
+	       Debug.log(1, "Enum skipping " + a);
+	       if (source[a] == ';')
+		 {
+		   Debug.log(1, "Found enum ;");
+		   endIndex = a + 1;
+		   break;
+		 }
+	       if (source[a] == '}')
+		 {
+		   Debug.log(1, "Found enum }");
+		   parser.classClosed();
+		   return a + 1;
+		 }
+	     }
+	   }
 	 int rc=parser.parse(source, endIndex, parser.getClassLevelComponents());
 	 return rc;
       }
@@ -708,6 +746,7 @@ public class Parser {
                                   String encoding, String expectedPackageName) 
       throws IOException, ParseException
    {
+     //System.err.println("Processing " + file + "...");
       this.currentFile = file;
       this.currentPackage = null;
       this.currentPackageName = null;
@@ -749,15 +788,15 @@ public class Parser {
             }
          }
        
-         /*
+         
            if (contextStack.size()>0) {
-           Debug.log(1,"-->contextStack not empty! size is "+contextStack.size());
+	     Debug.log(1,"-->contextStack not empty! size is "+contextStack.size());
            }
-         */
-  
+         
          return outerClass;
       }
       catch (IgnoredFileParseException ignore) {
+	Debug.log(1, "File ignored: " + ignore);
          return null;
       }
    }
@@ -770,22 +809,22 @@ public class Parser {
 	 int i=0;
 	 for (; i<componentTypes.length; ++i) {
 	    if ((match=componentTypes[i].match(source, index))>=0) {
-	       //Debug.log(9,componentTypes[i].getClass().getName()+" ("+match+"/"+source.length+")");
+	      //Debug.log(1,componentTypes[i].getClass().getName()+" ("+match+"/"+source.length+")");
 	       break;
 	    }
 	 }
 
 	 if (i<componentTypes.length) {
 	    int endIndex=componentTypes[i].getEndIndex(source, match);
-	    
+	    Debug.log(9, "Processing " + new String(source,index,endIndex-index) + " with " + componentTypes[i]);
 	    index=componentTypes[i].process(this, source, index, endIndex);
 	    if (index<0) {
-	       //Debug.log(9,"exiting parse because of "+componentTypes[i].getClass().getName()+" (\""+new String(source, index, endIndex-index)+"\")");
+	      //Debug.log(9,"exiting parse because of "+componentTypes[i].getClass().getName()+" (\""+new String(source, index, endIndex-index)+"\")");
 	       return endIndex;
 	    }
 	 }
 	 else {
-	    //Debug.log(9,"index="+index+", source.length()="+source.length);
+	   //Debug.log(9,"index="+index+", source.length()="+source.length);
 	    throw new ParseException("unmatched input in line "+currentLine+": "+new String(source, index, Math.min(50,source.length-index)));
 	 }
 
@@ -864,7 +903,6 @@ public class Parser {
       }
       classDoc.setImportedClasses((ClassDoc[])importedClassesList.toArray(new ClassDoc[0]));
 
-
       currentPackage.addClass(classDoc);
 
       currentClass = classDoc;
@@ -876,8 +914,8 @@ public class Parser {
       if (classDoc.superclass()!=null)
 	 referencedClassesList.add(classDoc.superclass());
 
-      //Debug.log(9,"classOpened "+classDoc+", adding superclass "+classDoc.superclass());
-
+      Debug.log(1,"classOpened "+classDoc+", adding superclass "+classDoc.superclass());
+      Debug.log(1,"Pushing " + ctx);
       contextStack.push(ctx);
       ctx=new Context(classDoc);
       //Debug.log(9,"ctx="+ctx);
@@ -890,7 +928,6 @@ public class Parser {
    }
 
    void classClosed() throws ParseException, IOException {
-
       ctx.classDoc.setFields((FieldDoc[])toArray(ctx.fieldList, 
                                                              new FieldDoc[0]));
       ctx.classDoc.setFilteredFields((FieldDoc[])toArray(ctx.filteredFieldList, 
@@ -914,10 +951,10 @@ public class Parser {
 	 }
       }
 
-      //Debug.log(9,"classClosed: "+ctx.classDoc);
+      Debug.log(1,"classClosed: "+ctx.classDoc);
 
       ctx=(Context)contextStack.pop();
-
+      Debug.log(1, "Popping " + ctx);
       ClassDoc[] referencedClasses=(ClassDoc[])referencedClassesList.toArray(new ClassDoc[0]);
 
       if (Main.DESCEND_SUPERCLASS) {
